#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 1.0
#pragma IgorVersion=5.0

Function/S GetApplicationData_dbf()
	
	// Path to file containing this function
	String theFilePath = FunctionPath("")

	String theSettingsFileName = "settings.txt"
	
	String theFullFilePath = ParseFilePath(1, theFilePath, ":", 1, 0) + theSettingsFileName
	
	String theReturnString = ReadTextFile(theFullFilePath)
		
	// return application
	return theReturnString
End

Function OpenProject_dbf(inProjectName)
	
	String inProjectName

	String thePathName = GetStandardProjectData(inProjectName, "PathName")
					
	String theFileName = GetStandardProjectData(inProjectName, "FileName")
	
	// open notebook
	OpenNotebook /K=1 /N=theSimdriveFile /P=$thePathName /V=0 theFileName
	
	// select input data
	Notebook theSimdriveFile selection={startOfFile, endOfFile }
	
	GetSelection notebook, theSimdriveFile , 2
	
	// copy input data
	String theInputData = S_Selection
	
	// close input file
	DoWindow /K  theSimdriveFile
	
	// get name of project window
	String theWinName = GetStandardProjectData(inProjectName, "WindowName")

	// display input file
	Notebook $theWinName, text = theInputData
	
	// get application
	String theApplication = GetStandardProjectData(inProjectName, "Application")
End

Function/S ProjectMenu_dbf()
	
	String theMenuString = "SimPost Help...\r"
	
	theMenuString += "-\r"
	
	String theCurrentProjectName = GetCurrentProjectName()
	
	String theProjectDF = GetStandardProjectData(theCurrentProjectName, "DataFolder")
	
	if (isEmdptDF(theProjectDF) == 1)
		
		theMenuString += "Load SIMDRIVE Results\r"
	else
		theMenuString += "(Load SIMDRIVE Results\r"
	endif

	theMenuString += "-\r"
		
	theMenuString += "Object View\r"

	theMenuString += "Type View\r"

	theMenuString += "SubType View\r"

	theMenuString += "-\r"

	String theReports = GetSimdriveReportList()
	
	Variable theSimdriveReports = ItemsInList(theReports)  
	
	Variable theReportIndex
	
	for (theReportIndex = 0; theReportIndex < theSimdriveReports; theReportIndex += 1)
	
		// extract report name
		String theReportName = StringFromList(theReportIndex, theReports)
	
		theMenuString += theReportName + "\r"		
	endfor 	
	
	return theMenuString
End

Function SimPostHelp()
	DisplayHelpTopic /K=1 "SimPost 1.0"
End

Function BeforeFileOpenHook_dbf(refNum,fileName,pathName,type,creator,kind)

	Variable refNum,kind	
	String fileName,pathName,type,creator

	// get path string
	PathInfo $pathName

	// full file path
	String theFullPath = S_Path + fileName

	// open project
	DoOpenProject(theFullPath)

	// load data
	LoadSIMDRIVEResults()

	// no user hook
	return 1
End

Function ProjectWinHook_deactivate_dbf(infoStr)

	String infoStr

	// no user hook
	return 1
End

Function OverwriteProject_dbf(inProjectName, inNewPath)

	String inProjectName, inNewPath
End

Function/S GetSimdriveReportList()
	
	// get function list
	String theFunctionList = FunctionList("*",";","KIND:2,NPARAMS:0")
	
	Variable theNumberOfFunctions = ItemsInList(theFunctionList)
	
	String theReportPath = ParseFilePath(1, FunctionPath("GetSimdriveReportList"), ":", 1, 0) + "reports:"
	
	String theReportList = "" 
	
	Variable theFunctionIndex
	
	for (theFunctionIndex = 0; theFunctionIndex < theNumberOfFunctions; theFunctionIndex += 1)
	
		String theFunction = StringFromList(theFunctionIndex, theFunctionList)
	
		String theFunctionPath = ParseFilePath(1, FunctionPath(theFunction), ":", 1, 0)

		if (stringmatch(theFunctionPath, theReportPath) == 1)
		
			// add function to report list
			theReportList += theFunction + ";"
		endif
	endfor 

	// return report list
	return theReportList
End

Function ObjectView()

	SetCurrentProjectView("ElementType;ObjectName;XDataType;Type;SubTypes")
End

Function TypeView()

	SetCurrentProjectView("ElementType;XDataType;Type;ObjectName;SubTypes")		
End

Function SubTypeView()

	SetCurrentProjectView("ElementType;XDataType;SubTypes;Type;ObjectName")
End

Function SetCurrentProjectView(inViewList)

	String inViewList
	
	String theBrowserInfo = GetBrowserSelection(-1)

	if (strlen(theBrowserInfo) > 0)

		Execute "ModifyBrowser collapse=0"
	endif
	
	DoUpdate
	DelayUpdate
	String theCurrentProject = GetCurrentProjectName()

	String theDataFolder = GetStandardProjectData(theCurrentProject, "DataFolder")	
		
	MoveWaveToProjectFolder(theCurrentProject)
	
	CreateView(theDataFolder, inViewList)
	
	if (strlen(theBrowserInfo) > 0)
		Execute "ModifyBrowser expand=0"
	endif
	
	Doupdate
End

Function MoveWaveToProjectFolder(inProjectName)

	String inProjectName
	
	String theDataFolder = GetStandardProjectData(inProjectName, "DataFolder")

	String theWaves = DoRecursiveWaveSearch(theDataFolder + ":", "")

	Variable theNumberOfWaves = ItemsInList(theWaves)
	Variable i

	for (i = 0; i < theNumberOfWaves; i+= 1)
	
		String theWave = StringFromList(i, theWaves)
		
		MoveWave $theWave, $(theDataFolder + ":")
	endfor

	String theObjectName = ""
	i = 0
	do
	
		theObjectName = GetIndexedObjName(theDataFolder, 4, 0 )
		
		if (strlen(theObjectName) > 0.5)
			KillDataFolder $(theDataFolder + ":'" + theObjectName + "'")
		else
		
			return 1
		endif
		
	while(1)
End

Function CreateView(inDataFolder, inKeyList)

	String inDataFolder, inKeyList
	
	String theNoKeyString = "no"
	
	SetDataFolder $inDataFolder 
	
	String theWaveList = WaveList("*",";", "")

	Variable theNumberOfKeys = ItemsInList(inKeyList)
	Variable theNumberOfWaves = ItemsInList(theWaveList)

	Variable i,j

	for (i = 0; i < theNumberOfWaves; i += 1)
	
		String theWave = StringFromList(i, theWaveList) 
		
		Wave theWaveRef = $(inDataFolder + ":" + theWave)
		
		String theWavesDataFolder = inDataFolder
		
		for (j = 0; j < theNumberOfKeys; j+= 1)
		
			String theKey = StringFromList(j, inKeyList)
			
			String theValue = GetWaveKey(inDataFolder + ":" + theWave, theKey)
			
			if (strlen(theValue) == 0)
				theValue = theNoKeyString + " " + theKey
			endif
			
			theWavesDataFolder += ":'" + CleanupName(theValue,1) + "'"

			NewDataFolder /O/S $theWavesDataFolder
		endfor
		
		MoveWave theWaveRef, $(theWavesDataFolder + ":")
	endfor
	
	SetDataFolder $inDataFolder
End

Function LoadSIMDRIVEResults()

	// get current project name
	String theProjectName = GetCurrentProjectName()

	// get base path
	String thePathName = GetStandardProjectData(theProjectName, "PathName")

	String theDataFolder = GetStandardProjectData(theProjectName, "DataFolder")

	SetDataFolder $(theDataFolder) 

	// load current project data
	DoLoadSimdriveData(thePathName,"")
End

Function Load_SIMDRIVE(thePathName, inKeyList)

	String thePathName, inKeyList

	DoLoadSimdriveData(thePathName, inKeyList)
End

Function DoLoadSimdriveData_(inPathName, inKeyList)
	
	String inPathName, inKeyList
	
	String theProjectName = GetCurrentProjectName()
	
	String theExistingWaves = GetProjectWaveList(theProjectName, "")
		
	String thePathName = inPathName
	
	String theFileName = GetStandardProjectData(theProjectName, "FileName")
	
	String theColorMap = GetStandardProjectData(theProjectName, "ColorMap")
		
	String theApplication = GetStandardProjectData(theProjectName, "Application")
	
	String theLineStyle = GetStandardProjectData(theProjectName, "LineStyle")

	String theLineWidth = GetStandardProjectData(theProjectName, "LineWidth")	
		
	String theFolderName = StringFromList(0,theFileName,".") + ".001"

	String theTempFolderName = "incoming"
	
	String theProjectDataFolder = GetDataFolder(-1)
	theProjectDataFolder = theProjectDataFolder[0, strlen(theProjectDataFolder)-2]
	
	String theFolderList = IndexedDir(thePathName, -1, 0)
		
	// build base note 0
	String theBaseNote_0 = "ProjectName=" + theProjectName + ";isProjectWave=True;"
	theBaseNote_0 += "Application=" + theApplication + ";LineStyle=" + theLineStyle + ";LineWidth="+ theLineWidth + ";"+ inKeyList
	
	// check result folder
	if (WhichListItem(theFolderName, theFolderList) > -1)
	
		PathInfo $thePathName
	
		String theResultPathString =   S_path + theFolderName
	
		NewPath /O /Q theResultPath ,  theResultPathString
		
		// get text file list
		String theTextFileList = IndexedFile(theResultPath, -1,".txt")
		
		String theSimDatFileNames = "Simulationsdaten.txt=German;Simulationdata.txt=English;"
		
		Variable theNumberOfSimDatFileNames = ItemsInList(theSimDatFileNames)
		
		String theLanguage = ""
		String theSimDatFileName = ""
		Variable theVersion = nan
		
		Variable theSimDatIndex = 0
		
		for (theSimDatIndex = 0; theSimDatIndex < theNumberOfSimDatFileNames; theSimDatIndex += 1)
			
			// get simdatdata
			String theSimDatData = StringFromList(theSimDatIndex, theSimDatFileNames)
				
			String theSimFileName = StringFromList(0, theSimDatData , "=")	
					
			String theLangStr = StringFromList(1, theSimDatData , "=")	
					
			if (WhichListItem(theSimFileName, theTextFileList) > 0)

				theLanguage = theLangStr
				theSimDatFileName = theSimFileName
				
				break
			endif
		endfor
		
		if (strlen(theLanguage) == 0 || strlen(theSimDatFileName) == 0)
			DoAlert 0, "Error loading simulationdata"
		endif

		// read simulation data file
		String theSimulationData = ReadTextFile(theResultPathString + ":" +  theSimDatFileName)
			
		String theFirstLine = StringFromList(0, theSimulationData, "\r")	
			
		// extract version information
		String theVersionInformation = StringFromList(0, theFirstLine , "(")	
	
		Variable theUsedSimdriveVersion = str2num(StringFromList(1, theVersionInformation, "\t"))
			
		// check version (!)	
		if (GetSimdriveVersion() < theUsedSimdriveVersion)
			DoAlert 0, "You are running Simdrive "+ num2str(theUsedSimdriveVersion) + "\rvisit: www.ina.com or www.wavemetrics.com\r to ckeck for new macros"
		endif	
			
		// get gid files
		String theGidFileList = IndexedFile(theResultPath, -1,".gid")

		theGidFileList = SortSimdriveFileList(theGidFileList)

		// get object name list
		String theObjectList = ""

		Variable theFileIndex
		
		Variable theNumberOfGidFiles = ItemsInList(theGidFileList)
		
		// loop to get all object names
		for (theFileIndex = 0; theFileIndex < theNumberOfGidFiles ; theFileIndex += 1)
		
			// get gig files
			String theGidFileName = StringFromList(theFileIndex, theGidFileList)
			
			if (!stringmatch(theGidFileName, "*_Z.gid"))
			
				String theObjectName = theGidFileName[0,strlen(theGidFileName)-5]
				
				// cleanup object name	
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = theObjectName[0, strlen(theObjectName)-2]

				// no global file, add object name to list
				theObjectList += theObjectName + ";"
			endif
		endfor
		
		// map Simdrive "SubTypes" to markers  
		String theSubtype2MarkerList = " minimum=-, maximum=+, Peak to Peak=p, spezifischer=s, integriert=I, bei maximaler Achskraft=+,"
		theSubtype2MarkerList+= " gemittelt=, averaged=, global=g, globaler=g,integrated=I, max.=+, specivic=s, at maximum Hubload=+"
		
		// load "*.gid" files
		for (theFileIndex = 0; theFileIndex < theNumberOfGidFiles ; theFileIndex += 1)
		
			// get file name
			theGidFileName = StringFromList(theFileIndex, theGidFileList)	
			
			if (!IsEmptyGidFile(theResultPathString + ":" + theGidFileName))
			
				// history
				print num2str(round((theFileIndex+1)/theNumberOfGidFiles*100)) + "%	processing: " + theGidFileName
			
				// global results or data vs time?
				if (stringmatch(theGidFileName,"*_Z.gid"))
			
					String theXDataType = "Engine Speed"
					theObjectName = theGidFileName[0,strlen(theGidFileName)-7]
				
					if (stringmatch(theLanguage, "German") == 1)
						String theXAxisLabel = "Drehzahl"
					else
						theXAxisLabel = "engine speed"
					endif
				else
					theXDataType = "Time"
					theObjectName = theGidFileName[0,strlen(theGidFileName)-5]
				
					if (stringmatch(theLanguage, "German") == 1)
						theXAxisLabel = "Zeit"
					else
						theXAxisLabel = "time"
					endif
				endif
				
				// cleanup object name	
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = theObjectName[0, strlen(theObjectName)-2]
		
				// build note
				String theBaseNote_1 = "ObjectName=" + theObjectName + ";XDataType=" + theXDataType + ";XAxisLabel=" + theXAxisLabel + ";"
				
				// read units
				String theUnitsList = ReadSimdriveHeader("theResultPath", theGidFileName,"EINHEIT = ['*")
				
				// read header
				String theWaveAndTypeNamesList = ReadSimdriveHeader("theResultPath", theGidFileName,"KANALNAME = ['*")
				
				// build wave name
				String theColumInfoStr = ""
				String theTypeList = ""
				String theSubTypeList = ""			
		
				Variable theWaveIndex
				
				for (theWaveIndex = 0; theWaveIndex < ItemsInList(theWaveAndTypeNamesList); theWaveIndex += 1)
				
					String theItem = StringFromList(theWaveIndex, theWaveAndTypeNamesList)
				
					String theType = StringFromList(0, theItem,",")
				
					theTypeList += theType + ";"
				
					// cean name
					String theCleanName = CleanupName(theType, 1)
				
					// check size
					if (strlen(theCleanName) > 27)
						theCleanName = theCleanName[0,26]
					endif
					
					// add id
					theCleanName += " " + num2str(theWaveindex)
										
					// build colum info str
					theColumInfoStr += "N='" + theCleanName + "';"
					
					// do we have subtypes?
					if (ItemsInList(theItem,",") > 1)
						theSubTypeList += RemoveListItem(0,theItem,",") + ";"
					else
						// insert space for no subtype
						theSubTypeList += " ;"
					endif
				endfor
				
				String theElementType = GetElementTypeString(GetIDFromFileName(theGidFileName, 0))

				// create new data folder
				NewDataFolder/O /S  $(theProjectDataFolder + ":" + theTempFolderName)
																
				// check for data
				if (strlen(theColumInfoStr) > 0)	
				
					// read data
					LoadWave/A /W /B=theColumInfoStr /D /G /Q /O /P=theResultPath theGidFileName
				endif

				// first wave is x - wave
				String theXWaveName = StringByKey("N", StringFromList(0,theColumInfoStr,";"),"=",";")	
				String theXWaveUnit = StringFromList(0, theUnitsList)
				
				// last loop to set wave notes!
				for (theWaveIndex = 0; theWaveIndex < ItemsInList(theColumInfoStr); theWaveIndex += 1)

					// extract item
					String theInfoStr = StringFromList(theWaveIndex,theColumInfoStr)
					
					// get name
					String theName = StringByKey("N",theInfoStr,"=",";")					
					
					// get type
					theType = StringFromList(theWaveIndex,theTypeList)
					
					String theSubType = StringFromList(theWaveIndex,theSubTypeList)
				
					if (stringmatch(theSubType, " ") == 1)
						theSubType = ""
					endif
					
					// get standard
					String theMarker = GetProjectWaveMarker(theSubtype2MarkerList, theSubType, ",")
					
					// get y-unit
					String theUnit = StringFromList(theWaveIndex,theUnitsList)
				
					// legend					
					String theLegendString = theObjectName
					if (strlen(theSubType) > 1)
					
						// add sub type
				 		theLegendString +=  "," + theSubType 
					endif
					
					// label
					String theLabel = theType
								
					// get help topic from type string
					String theHelpTopic = 	StringFromList(0, theType," ")
										
					// build wave note
					String theBaseNote_2 = "WaveName=" + theName + ";Type=" + theType + ";SubTypes=" + theSubType + ";Unit=" + theUnit + ";ElementType=" + theElementType + ";"
					theBaseNote_2 += "Marker=" + theMarker + ";Legend=" + theLegendString + ";YAxisLabel=" + theLabel + ";XWaveName=" + theXWaveName + ";Language=" + theLanguage + ";XWaveUnit=" + theXWaveUnit + ";"

					// first wave
					if (theWaveIndex == 0)
						theBaseNote_2 += "IsYData=false;" 
					else
						theBaseNote_2 += "IsYData=true;"
					endif

					// set unit
					SetScale /P y, 0, 0 ,theUnit $(theName[1,strlen(theName)-2])
	
					// sort note
					String theSortedNote = SortList(theBaseNote_0 + theBaseNote_1 + theBaseNote_2 , ";")
				
					// set wave note
					Note $(theName[1,strlen(theName)-2]) , theSortedNote
				
					Rename $(theName[1,strlen(theName)-2]), $UniqueName("_tempXID", 1, 0)
				endfor	
			endif
		endfor
	endif

	String theIncomingWaves = WaveList("*", ";", "")

	Variable theNumberOfIncomingWaves = ItemsInList(theIncomingWaves)

	Variable theIncomIndex
	
	String theAllObjectsList = ""
	
	// log 
	String theNewWaves = ""
	String theOverwrittenWaves = ""
	String theErrorWaves = ""
	
	// overwrite waves
	for (theIncomIndex = 0; theIncomIndex < theNumberOfIncomingWaves; theIncomIndex += 1)
	
		// get wave
		String theIncomTempWave = StringFromList(theIncomIndex, theIncomingWaves)

		String theIncomWave = StringFromList(1, theIncomTempWave, "X")

		String theSearchString = "*':'" + theIncomWave + "'"

		String theExistingWave = ListMatch(theExistingWaves, theSearchString, ";" )

		Variable theNumberOfExistingWaves = ItemsInList(theExistingWave)

		switch (theNumberOfExistingWaves)
		
			case 0:
				
				// new wave
				theNewWaves += theIncomWave + ";"

				MoveWave $theIncomTempWave, $(theProjectDataFolder + ":" + theIncomWave)
			break
			
			case 1:
			
				// overwrite existing wave
				theExistingWave = StringFromList(0, theExistingWave)
				Duplicate/O $theIncomTempWave, $theExistingWave
				
				theOverwrittenWaves += theIncomWave + ";"
				
				Killwaves $theIncomTempWave
			break
			
				// error
				theErrorWaves += theIncomWave + ":"
			default :
			
		endswitch		
	endfor
	
	// get all project waves
	String theTotalProjectWaves = GetProjectWaveList(theProjectName, "")
	
	// output to history
	Print "*** Load Report ***\r"
	Print "project : " + theProjectName + "\r"
	Print "path : " + theResultPathString + "\r"
	Print "data folder : " + theProjectDataFolder + "\r"
	Print "date = " + Date()
	Print "time = " + Time()	
	Print "---\r"
	Print "> total waves : " + num2str(ItemsInList(theTotalProjectWaves) )+ " <\r"
	Print "> new waves : " + num2str(ItemsInList(theNewWaves) ) + " <\r"
	Print "> overwritten waves  : " + num2str(ItemsInList(theOverwrittenWaves) ) + " <\r"
	Print "> errors loading waves  : " + num2str(ItemsInList(theErrorWaves)) + "< \r"
	Print "*** Load Report End ***\r"
	
	// set project DF
	SetDataFolder $(theProjectDataFolder) 
	
	KillEmptyDF(theProjectDataFolder) 

	SetCurrentProjectView("ElementType;ObjectName;XDataType;Type;SubTypes")

	// cleanup
	KillPath theResultPath
End

Function/S SortSimdriveFileList(inGidFileList)

	String inGidFileList 
			
	String theGidFileList = inGidFileList
	
	// build new list
	Variable theGidFileIndex
		
	Variable theNumberOfGidFiles = ItemsInList(theGidFileList)
	
	// sort list
	String theSortedList = ""
	Variable theElementIndex = 0
		
	do
		
		theSortedList +=  ListMatch(theGidFileList, "*_" + num2str(theElementIndex)+ ".gid") + ListMatch(theGidFileList, "*_" + num2str(theElementIndex)+ "_Z.gid")
		theElementIndex += 1
	
	while(ItemsInList(theSortedList) != theNumberOfGidFiles && theElementIndex < 200)
	
	return theSortedList
End

Function/S ReadSimdriveHeader(inPathName, inFileName, inMatchString)
	
	// Simdrive file
	String inPathName, inFileName, inMatchString
	String theData = ""

	Variable refNum, colum, lineCount
	String str
		
	// Open file for read.
	Open/R/Z=2 /P=$inPathName refNum as inFileName

	// store results from Open in a safe place.
	Variable err = V_flag
	String fullPath = S_fileName
 
	String theHeader = ""

	do
		// read 
		FReadLine refNum, str
			
		// line found?	
		if(stringmatch(str, inMatchString))

			// store names
			theHeader = StringFromList(0,StringFromList(1,str,"["),"]")
		endif
				
		// increase line counter
		lineCount  = lineCount +1
		 	
	while(strlen(theHeader) == 0)
	
	// clean string
	Variable j
	String theList = ""
	for(j = 1; j < ItemsInList(theHeader,"'") ; j +=2)
	
		// get name
		String theItem = StringFromList(j,theHeader,"'")
		
		// add
		theList += theItem + ";"
	endfor
	
	// close file
	close refNum

	// return list
	return theList
End

Function/S GetElementTypeString(inElementTypeNumber)
	
	Variable inElementTypeNumber
		
	// Path to file containing this function
	String theFilePath = FunctionPath("")

	String theSettingsFileName = "types.txt"
	
	String theFullFilePath = ParseFilePath(1, theFilePath, ":", 1, 0) + theSettingsFileName
	
	String theTypeStringList = ReadTextFile(theFullFilePath)
	
	String theName = StringByKey(num2str(inElementTypeNumber), theTypeStringList , "=", "\r")

	if (strlen(theName) == 0)
		theName = "ElementType=" + num2str(inElementTypeNumber)
	endif

	return theName
End

Function GetIDFromFileName(inFileName, inIdIndex)

	String inFileName
	Variable inIdIndex
	
	if (stringmatch(inFileName, "*_Z.gid") == 1)
		
		inFileName = inFileName[0, strlen(inFileName)-7]

	else
			inFileName = inFileName[0, strlen(inFileName)-5]
	endif

	Variable theID = str2num(StringFromList(ItemsInList(inFileName, "_")-inIdIndex-1, inFileName, "_"))
	
	return theID
End

Function/S GetProjectWaveMarker(inSubType2MarkerList, inSubTypeList, inSeperatorString)

	String inSubType2MarkerList, inSubTypeList, inSeperatorString

	String theMarkers = ""
	
	Variable theSubTypeIndex
	
	for (theSubTypeIndex = 0; theSubTypeIndex < ItemsInList(inSubTypeList, inSeperatorString); theSubTypeIndex += 1)
	
		String theSubType = StringFromList(theSubTypeIndex,inSubTypeList, inSeperatorString)
		
		// try to get a marker
		String theMarker = StringByKey(theSubType,inSubtype2MarkerList,"=", inSeperatorString)
		
		if (strlen(theMarker) > 0)
			
			// add
			theMarkers += theMarker
		endif
	endfor

	// return standard marker
	return theMarkers
End

Function KillEmptyDF(inDataFolder)

	String inDataFolder
	
	String theKillList = ""
	
	String objName
	Variable index = 0
	do
		objName = GetIndexedObjName(inDataFolder, 4, index)
		if (strlen(objName) == 0)
			break
		endif
		
		String theFullName = inDataFolder + ":'" + objName + "':"
		
		Variable theObjectType, theNumberofObjects = 0
		
		for (theObjectType = 1; theObjectType <=4; theObjectType += 1)
		
			theNumberofObjects += CountObjects(theFullName, theObjectType)
		endfor 
		
		if (theNumberofObjects == 0)
			theKillList += theFullName + ";"
		endif
		
		index += 1
	while(1)
	
	for (index = 0; index < ItemsInList(theKillList); index += 1)
	
		KillDataFolder $StringFromList(index, theKillList)
	endfor
End

Function GetSimdriveVersion()
	return 4.0
End

Function IsEmptyGidFile(inGidFile)

	String inGidFile
		
	String str
	
	Variable refNum
	
	// Open file for read.
	Open/R/Z=2 refNum as inGidFile

	Variable theCounter = 0
	Variable theEndLine = nan
	
	do
		theCounter += 1
	
		FReadLine refNum, str
		
		Variable theLength = strlen(str)
		
		if (stringmatch("END\r", str) == 1)
			
			theEndLine = theCounter
		endif
		
		if (theLength > 0 && theEndLine != NaN && theCounter - theEndLine > 1 )
			return 0
		endif
		
	while(thelength > 0)
	
	Close refNum
	
	return 1
End

Function DoLoadSimdriveData(inPathName, inKeyList)
	
	String inPathName, inKeyList
	
	Variable theStartTime = DateTime
	
	String theProjectName = GetCurrentProjectName()
	
	String theExistingWaves = GetProjectWaveList(theProjectName, "")
		
	String thePathName = inPathName
	
	String theFileName = GetStandardProjectData(theProjectName, "FileName")
	
	String theColorMap = GetStandardProjectData(theProjectName, "ColorMap")
		
	String theApplication = GetStandardProjectData(theProjectName, "Application")
	
	String theLineStyle = GetStandardProjectData(theProjectName, "LineStyle")

	String theLineWidth = GetStandardProjectData(theProjectName, "LineWidth")	
		
	String theFolderName = StringFromList(0,theFileName,".") + ".001"

	String theTempFolderName = "incoming"
	
	String theProjectDataFolder = GetDataFolder(-1)
	theProjectDataFolder = theProjectDataFolder[0, strlen(theProjectDataFolder)-2]
	
	String theFolderList = IndexedDir(thePathName, -1, 0)
		
	if (isEmdptDF(theProjectDataFolder) == 0)
		Abort "Error loading data.\rData folder :" + theProjectDataFolder + " must be empty."
	endif	
		
	// build base note 0
	String theBaseNote_0 = "ProjectName=" + theProjectName + ";isProjectWave=True;"
	theBaseNote_0 += "Application=" + theApplication + ";LineStyle=" + theLineStyle + ";LineWidth="+ theLineWidth + ";"+ inKeyList
	
	Variable theWaveCounter = 0
	
	// check result folder
	if (WhichListItem(theFolderName, theFolderList) > -1)
	
		PathInfo $thePathName
	
		String theResultPathString =   S_path + theFolderName
	
		NewPath /O /Q theResultPath ,  theResultPathString
		
		// get text file list
		String theTextFileList = IndexedFile(theResultPath, -1,".txt")
		
		String theSimDatFileNames = "Simulationsdaten.txt=German;Simulationdata.txt=English;"
		
		Variable theNumberOfSimDatFileNames = ItemsInList(theSimDatFileNames)
		
		String theLanguage = ""
		String theSimDatFileName = ""
		Variable theVersion = nan
		
		Variable theSimDatIndex = 0
		
		for (theSimDatIndex = 0; theSimDatIndex < theNumberOfSimDatFileNames; theSimDatIndex += 1)
			
			// get simdatdata
			String theSimDatData = StringFromList(theSimDatIndex, theSimDatFileNames)
				
			String theSimFileName = StringFromList(0, theSimDatData , "=")	
					
			String theLangStr = StringFromList(1, theSimDatData , "=")	
					
			if (WhichListItem(theSimFileName, theTextFileList) > 0)

				theLanguage = theLangStr
				theSimDatFileName = theSimFileName
				
				break
			endif
		endfor
		
		if (strlen(theLanguage) == 0 || strlen(theSimDatFileName) == 0)
			DoAlert 0, "Error loading simulationdata"
		endif

		// read simulation data file
		String theSimulationData = ReadTextFile(theResultPathString + ":" +  theSimDatFileName)
			
		String theFirstLine = StringFromList(0, theSimulationData, "\r")	
			
		// extract version information
		String theVersionInformation = StringFromList(0, theFirstLine , "(")	
	
		Variable theUsedSimdriveVersion = str2num(StringFromList(1, theVersionInformation, "\t"))
			
		// check version (!)	
		if (GetSimdriveVersion() < theUsedSimdriveVersion)
			DoAlert 0, "You are running Simdrive "+ num2str(theUsedSimdriveVersion) + "\rvisit: www.ina.com or www.wavemetrics.com\r to ckeck for new macros"
		endif	
			
		// get gid files
		String theGidFileList = IndexedFile(theResultPath, -1,".gid")

		theGidFileList = SortSimdriveFileList(theGidFileList)

		Variable theFileIndex
		
		Variable theNumberOfGidFiles = ItemsInList(theGidFileList)

		// check output format	
		if (theNumberOfGidFiles == 0)
			DoAlert 0, "No *.gid files found in "+ theFolderName + "\rcheck Simdrive output format"
		endif	
		
		// map Simdrive "SubTypes" to markers  
		String theSubtype2MarkerList = " minimum=-, maximum=+, Peak to Peak=p, spezifischer=s, integriert=I, bei maximaler Achskraft=+,"
		theSubtype2MarkerList+= " gemittelt=, averaged=, global=g, globaler=g,integrated=I, max.=+, specivic=s, at maximum Hubload=+"
		
		// history
		Print "*** Load Report ***\r"
		Print "project		: " + theProjectName + "\r"
		Print "path		: " + theResultPathString + "\r"
		Print "data folder	: " + theProjectDataFolder + "\r"
		Print "\rloading ...\r"
		
		// load "*.gid" files
		for (theFileIndex = 0; theFileIndex < theNumberOfGidFiles ; theFileIndex += 1)
		
			// get file name
			String theGidFileName = StringFromList(theFileIndex, theGidFileList)	
			
			if (!IsEmptyGidFile(theResultPathString + ":" + theGidFileName))
			
				// history
				print "progress: " + num2str(round((theFileIndex+1)/theNumberOfGidFiles*100)) + "%	=>	processing: " + theGidFileName
			
				// global results or data vs time?
				if (stringmatch(theGidFileName,"*_Z.gid"))
			
					String theXDataType = "Engine Speed"
					String theObjectName = theGidFileName[0,strlen(theGidFileName)-7]
				
					if (stringmatch(theLanguage, "German") == 1)
						String theXAxisLabel = "Drehzahl"
					else
						theXAxisLabel = "engine speed"
					endif
				else
					theXDataType = "Time"
					theObjectName = theGidFileName[0,strlen(theGidFileName)-5]
				
					if (stringmatch(theLanguage, "German") == 1)
						theXAxisLabel = "Zeit"
					else
						theXAxisLabel = "time"
					endif
				endif
				
				// cleanup object name	
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = RemoveListItem(ItemsInList(theObjectName, "_")-1, theObjectName, "_")
				theObjectName = theObjectName[0, strlen(theObjectName)-2]
		
				// build note
				String theBaseNote_1 = "ObjectName=" + theObjectName + ";XDataType=" + theXDataType + ";XAxisLabel=" + theXAxisLabel + ";"
				
				// read units
				String theUnitsList = ReadSimdriveHeader("theResultPath", theGidFileName,"EINHEIT = ['*")
				
				// read header
				String theWaveAndTypeNamesList = ReadSimdriveHeader("theResultPath", theGidFileName,"KANALNAME = ['*")
				
				// build wave name
				String theColumInfoStr = ""
				String theTypeList = ""
				String theSubTypeList = ""			
		
				Variable theWaveIndex
				
				Variable theNumberOfTypeAndWaves = ItemsInList(theWaveAndTypeNamesList)
				
				for (theWaveIndex = 0; theWaveIndex < theNumberOfTypeAndWaves; theWaveIndex += 1)
				
					String theItem = StringFromList(theWaveIndex, theWaveAndTypeNamesList)
				
					String theType = StringFromList(0, theItem,",")
				
					theTypeList += theType + ";"													
	
					// clean name
					String theCleanName = theObjectName + "_"
															
					String theWavesSubType = RemoveListItem(0,theItem,",")
					
					// do we have subtypes?
					if (ItemsInList(theItem,",") > 1)
						theSubTypeList += theWavesSubType + ";"
						theCleanName += theWavesSubType + "_"
					else
						// insert space for no subtype
						theSubTypeList += " ;"
					endif

					// clean name
					theCleanName = CleanupName(theCleanName  , 0)

					// check size
					if (strlen(theCleanName) > 27)
						theCleanName = theCleanName[0,26]
					endif
					
					// add ID
					theCleanName +=  num2str(theWaveCounter)
	
					// build colum info str
					theColumInfoStr += "N='" + theCleanName + "';"
					
					theWaveCounter += 1
				endfor
				
				String theElementType = GetElementTypeString(GetIDFromFileName(theGidFileName, 0))

				// create new data folder
				NewDataFolder/O /S  $(theProjectDataFolder + ":" + theTempFolderName)

				// create new data folder
				NewDataFolder/O /S  $(theProjectDataFolder + ":" + theTempFolderName + ":'" + theElementType + "'")

				// create new data folder
				NewDataFolder/O /S  $(theProjectDataFolder + ":" + theTempFolderName + ":'" + theElementType + "':'" + theObjectName + "'")
										
				// check for data
				if (strlen(theColumInfoStr) > 0)	
				
					// read data
					LoadWave/A /W /B=theColumInfoStr /D /G /Q /O /P=theResultPath theGidFileName
				endif

				// first wave is x - wave
				String theXWaveName = StringByKey("N", StringFromList(0,theColumInfoStr,";"),"=",";")	
				String theXWaveUnit = StringFromList(0, theUnitsList)
				
				Variable theNumberOfItems = ItemsInList(theColumInfoStr)
				
				// last loop to set wave notes!
				for (theWaveIndex = 0; theWaveIndex < theNumberOfItems; theWaveIndex += 1)

					// extract item
					String theInfoStr = StringFromList(theWaveIndex,theColumInfoStr)
					
					// get name
					String theName = StringByKey("N",theInfoStr,"=",";")					
					
					// get type
					theType = StringFromList(theWaveIndex,theTypeList)
					
					String theSubType = StringFromList(theWaveIndex,theSubTypeList)
				
					if (stringmatch(theSubType, " ") == 1)
						theSubType = ""
					endif
					
					// get standard
					String theMarker = GetProjectWaveMarker(theSubtype2MarkerList, theSubType, ",")
					
					// get y-unit
					String theUnit = StringFromList(theWaveIndex,theUnitsList)
				
					// legend					
					String theLegendString = theObjectName
					if (strlen(theSubType) > 1)
					
						// add sub type
				 		theLegendString +=  "," + theSubType 
					endif
					
					// label
					String theLabel = theType
								
					// get help topic from type string
					String theHelpTopic = 	StringFromList(0, theType," ")
										
					// build wave note
					String theBaseNote_2 = "WaveName=" + theName + ";Type=" + theType + ";SubTypes=" + theSubType + ";Unit=" + theUnit + ";ElementType=" + theElementType + ";"
					theBaseNote_2 += "Marker=" + theMarker + ";Legend=" + theLegendString + ";YAxisLabel=" + theLabel + ";XWaveName=" + theXWaveName + ";Language=" + theLanguage + ";XWaveUnit=" + theXWaveUnit + ";"

					// first wave
					if (theWaveIndex == 0)
						theBaseNote_2 += "IsYData=false;" 
					else
						theBaseNote_2 += "IsYData=true;"
					endif

					// set unit
					SetScale /P y, 0, 0 ,theUnit $(theName[1,strlen(theName)-2])
	
					// sort note
					String theSortedNote = SortList(theBaseNote_0 + theBaseNote_1 + theBaseNote_2 , ";")
				
					// set wave note
					Note $(theName[1,strlen(theName)-2]) , theSortedNote
				endfor	
			endif
		endfor
	endif

	String theIncomingWaves = DoRecursiveWaveSearch(theProjectDataFolder + ":" + theTempFolderName + ":", "WaveName=*")

	Variable theNumberOfIncomingWaves = ItemsInList(theIncomingWaves)

	Variable theIncomIndex
	
	String theAllObjectsList = ""
	
	// color loop
	for (theIncomIndex = 0; theIncomIndex < theNumberOfIncomingWaves; theIncomIndex += 1)
	
		// get wave
		String theWave = StringFromList(theIncomIndex, theIncomingWaves)
		
		String theWavesDataFolder = GetWavesDataFolder($theWave, 0)
		
		theNumberOfItems = ItemsInList(theWave, ":")
		
		String theDataFolder = RemoveListItem(theNumberOfItems-1, theWave, ":")
		theDataFolder = RemoveListItem(theNumberOfItems-2, theDataFolder, ":")
		
		Variable theNumberOfDataFolders = CountObjects(theDataFolder, 4)
		
		Variable theDFIndex
		
		String theDFNames = ""
		
		for (theDFIndex = 0; theDFIndex < theNumberOfDataFolders; theDFIndex += 1)
		
			 theDFNames += GetIndexedObjName(theDataFolder, 4, theDFIndex ) + ";"
		endfor

		String theColor = GetStandardObjectColor(theWavesDataFolder, theDFNames, theColorMap)
		
		String theNote = note($theWave)
		
		Note $theWave, theNote + "Color=" + theColor + ";"
	endfor
	
	// log 
	String theNewWaves = ""
	String theOverwrittenWaves = ""
	String theErrorWaves = ""
	
	// get all project waves
	String theTotalProjectWaves = GetProjectWaveList(theProjectName, "")
	
	// output to history
	Print "-------- done -------\r"
	Print "> total waves : " + num2str(ItemsInList(theTotalProjectWaves) )+ " <\r"
	Print "> new waves : " + num2str(ItemsInList(theNewWaves) ) + " <\r"
	Print "> overwritten waves  : " + num2str(ItemsInList(theOverwrittenWaves) ) + " <\r"
	Print "> errors loading waves  : " + num2str(ItemsInList(theErrorWaves)) + "< \r"
	Print "> cpu time  : " + num2str(DateTime - theStartTime) + " [s] < \r"	
	Print "*** Load Report End ***\r"
	
	// set project DF
	SetDataFolder $(theProjectDataFolder) 
	
	KillEmptyDF(theProjectDataFolder) 

	SetCurrentProjectView("ElementType;ObjectName;XDataType;Type;SubTypes")

	// cleanup
	KillPath theResultPath
End

Function isEmdptDF(inDF)

	String inDF
	
	Variable i
	
	for (i = 0; i < 5; i += 1)

		if (CountObjects(inDF, i) > 0)
			return 0
		endif
	endfor
	
	return 1
End